import sys
import subprocess
import os
from pathlib import Path
import shutil


def read_cli_args(args):
    if not args:
        print("❌ Project name is missing. Usage: qa-gen <project-name> [--force] [--dry-run] [--path <destination>]")
        sys.exit(2)

    project_name = args[0]
    force = False
    dry = False
    path = "."

    i = 1
    while i < len(args):
        if args[i] == "--force":
            force = True
        elif args[i] == "--dry-run":
            dry = True
        elif args[i] == "--path" and i + 1 < len(args):
            path = args[i + 1]
            i += 1
        i += 1

    print(f"Project: {project_name} | Force: {force} | Dry-run: {dry} | Path: {path}")
    return project_name, force, dry, path


def resolve_cmd(cmd_name):
    path = shutil.which(cmd_name)
    if path is None:
        print(f"❌ '{cmd_name}' is not found in your PATH.")
        print(f"🔍 Current PATH: {os.environ.get('PATH')}")
        sys.exit(1)
    return path

def check_prerequisites():
    node_cmd = resolve_cmd("node")
    npm_cmd = resolve_cmd("npm")

    try:
        node_check = subprocess.run(
            [node_cmd, "--version"],
            capture_output=True,
            text=True,
            check=True,
            env=os.environ.copy(),  # ✅ Use full shell env for WSL/NVM/Git Bash
        )
        node_check_result = node_check.stdout.strip()
        major = int(node_check_result.lstrip("v").split(".")[0])
        if major < 18:
            print("❌ Node.js 18+ required.")
            sys.exit(1)
        print(f"✅ Checked Node.js: {node_check_result}")

    except (FileNotFoundError, subprocess.CalledProcessError) as e:
        print("❌ Failed to run Node.js command.")
        print(str(e))
        sys.exit(1)

    try:
        npm_check = subprocess.run(
            [npm_cmd, "--version"],
            capture_output=True,
            text=True,
            check=True,
            env=os.environ.copy(),
        )
        print(f"✅ Checked npm: {npm_check.stdout.strip()}")

    except (FileNotFoundError, subprocess.CalledProcessError) as e:
        print("❌ Failed to run npm command.")
        print(str(e))
        sys.exit(1)

    return

def create_folders(project_name, dry_run=False):
    base = Path(project_name)
    relative_paths = [
        "tests/e2e",
        "tests/front_end/feature_one",
        "tests/back_end/feature_one",
        "tests/support/POMs/feature_one",
        "tests/support/utils",
    ]

    if dry_run:
        for path in relative_paths:
            folder_path = base / path
            print(f"⚙️ [DRY RUN] would create: < {folder_path} >")
        return
    else:
        for path in relative_paths:
            folder_path = base / path
            folder_path.mkdir(parents=True, exist_ok=True)
            print(f"📥 Created: < {folder_path} >")
        return

def ensure_root(project_name, dry_run=False):
    base = Path(project_name)
    if dry_run:
        print("⚙️ [DRY RUN] Would create root folder if missing:", base)
    else:
        if not base.exists():
            base.mkdir(parents=True)
            print("📥 Created:", base)
        else:
            print("✅ Exists:", base)
    return base


def init_node_project(project_root, dry_run=False):
    project_root = Path(project_root)
    if dry_run:
        print("⚙️ [DRY RUN] Would run: npm init -y")
        print("⚙️ [DRY RUN] Would run: npm install -D playwright @playwright/test typescript ts-node")
        if sys.platform.startswith("linux"):
            print("⚙️ [DRY RUN] Would run: npx playwright install --with-deps")
        else:
            print("⚙️ [DRY RUN] Would run: npx playwright install")
    else:
        try:
            subprocess.run(["npm", "init", "-y"], cwd=project_root, check=True)
            print("✅ Node.js project initialized.")
        
        except FileNotFoundError as e:
            print(f"❌ Error: {e}")
            sys.exit(1)
        except subprocess.CalledProcessError as e:
            print(f"❌ Error: {e.stderr}")
            sys.exit(1)

        try:
            subprocess.run(["npm", "install", "-D", "@faker-js/faker"], cwd=project_root, check=True)
            print("✅ Faker.JS installed and ready to use.")
        
        except FileNotFoundError as e:
            print(f"❌ Error: {e}")
            sys.exit(1)
        except subprocess.CalledProcessError as e:
            print(f"❌ Error: {e.stderr}")
            sys.exit(1)

        try:
            subprocess.run(["npm", "install", "dotenv"], cwd=project_root, check=True)
            print("✅ DOTENV installed and ready to use.")
        
        except FileNotFoundError as e:
            print(f"❌ Error: {e}")
            sys.exit(1)
        except subprocess.CalledProcessError as e:
            print(f"❌ Error: {e.stderr}")
            sys.exit(1)

        try:
            subprocess.run(["npm", "install", "-D", "playwright", "@playwright/test", "typescript", "ts-node"], cwd=project_root, check=True)
            print("✅ Playwright is now installed.")

        except FileNotFoundError as e:
            print(f"❌ Error: {e}")
            sys.exit(1)
        except subprocess.CalledProcessError as e:
            print(f"❌ Error: {e.stderr}")
            sys.exit(1)

        try:
            if sys.platform.startswith("linux"):
                subprocess.run(["npx", "playwright", "install", "--with-deps"], cwd=project_root, check=True)
                print("✅ Playwright system deps + browsers installed.")
            else:
                subprocess.run(["npx", "playwright", "install"], cwd=project_root, check=True)
                print("✅ Playwright browsers are now installed.")

        except FileNotFoundError as e:
            print(f"❌ Error: {e}")
            sys.exit(1)
        except subprocess.CalledProcessError as e:
            print(f"❌ Error: {e.stderr}")
            sys.exit(1)
    
def remove_node_project(project_root, dry_run=False):
    project_root = Path(project_root)

    targets = [
        project_root / "node_modules",
        project_root / "package.json",
        project_root / "package-lock.json",
        project_root / "tsconfig.json",
        project_root / "playwright.config.ts",
        project_root / "tests",
        project_root / ".gitignore",
        project_root / "README.md"
    ]

    for target in targets:
        if dry_run:
            print(f"⚙️ [DRY RUN] Would remove: {target}")
        else:
            if target.is_dir():
                shutil.rmtree(target, ignore_errors=True)
                print(f"🗑️ Removed directory: {target}")
            elif target.is_file():
                target.unlink(missing_ok=True)
                print(f"🗑️ Removed file: {target}")
            else:
                print(f"ℹ️ Nothing to remove at: {target}")

        







from pathlib import Path
from jinja2 import Environment, FileSystemLoader

TEMPLATE_DIR = Path(__file__).parent / "templates"

env = Environment(loader=FileSystemLoader(str(TEMPLATE_DIR)))



def render_template(template_name: str, dest_path: Path, context=None, dry_run=False, force=False):
    context = context or {}

    if dry_run:
        print(f"⚙️ [DRY RUN] Would render template: {template_name} → {dest_path}")
        return

    template = env.get_template(template_name)
    content = template.render(**context)

    dest_path.parent.mkdir(parents=True, exist_ok=True)

    if dest_path.exists() and not force:
        print(f"⏭️ Skipped (exists): {dest_path}")
        return

    dest_path.write_text(content, encoding="utf-8")
    if dest_path.exists() and force:
        print(f"✏️ Overwrote: {dest_path}")
    else:
        print(f"📝 Rendered: {dest_path}")


def create_files_from_templates(project_root: Path, dry_run=False, force=False, context=None):
    context = context or {}
    templates = {
        "playwright.config.ts.j2": "playwright.config.ts",
        "tsconfig.json.j2": "tsconfig.json",
        ".gitignore.j2": ".gitignore",
        "README.md.j2": "README.md",
        "tests/e2e/e2e_example.spec.ts.j2": "tests/e2e/e2e_example.spec.ts.j2",
        "tests/front_end/feature_one/example_frontend.spec.ts.j2": "tests/front_end/feature_one/example_frontend.spec.ts",
        "tests/back_end/feature_one/example_backend.spec.ts.j2": "tests/back_end/feature_one/example_backend.spec.ts",
        "tests/support/POMs/feature_one/example.page.ts.j2": "tests/support/POMs/feature_one/example.page.ts",
        "tests/support/utils/helpers.ts.j2": "tests/support/utils/helpers.ts",
        "tests/support/utils/test-data.ts.j2": "tests/support/utils/test-data.ts",
        "tests/support/utils/fixtures.ts.j2": "tests/support/utils/fixtures.ts"
    }

    for template_file, output_file in templates.items():
        render_template(template_file, project_root / output_file, context, dry_run, force)



from .project_setup_helpers import *
from .files_setup_helpers import *
from pathlib import Path
import sys

def main():
    project_name, force, dry_run, custom_path = read_cli_args(sys.argv[1:])
    project_root = Path(custom_path) / project_name

    if "--clean" in sys.argv[1:]:
        remove_node_project(project_root, dry_run)
        return

    check_prerequisites()
    project_root = ensure_root(project_root, dry_run)
    init_node_project(project_root, dry_run)
    create_folders(project_root, dry_run)



    create_files_from_templates(project_root, dry_run, force, context={"project_name": project_name})



    print("\n🚀 Done! Your Playwright testing project is ready.")
    if dry_run:
        print("🧪 This was a dry run. No actual files were written.")
    else:
        print(f"📁 Navigate to: {project_name}")
        print("💡 Run tests with: `npx playwright test`")

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name='qa-project-gen',
    version='0.1.4',
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'jinja2',
    ],
    entry_points={
        'console_scripts': [
            'qa-gen = qa_project_gen.main:main',
        ],
    },
        package_data={
            'qa_project_gen': [
                'templates/*.j2',
                'templates/*/*.j2',
                'templates/*/*/*.j2',
                'templates/*/*/*/*.j2',
            ]
}
)
